@@ -1,8 +1,16 @@
+.msg <- message
+
 .onLoad <-
 function(libname, pkgname) {
     OPATH <- Sys.getenv("PATH")
     javahome <- if (!is.null(getOption("java.home"))) getOption("java.home") else Sys.getenv("JAVA_HOME")
-    if(!nchar(javahome)) { ## JAVA_HOME was not set explicitly
+    if (nzchar(javahome) && !dir.exists(javahome)) {
+         .msg("java.home option: ", getOption("java.home"))
+         .msg("JAVA_HOME environment variable: ", Sys.getenv("JAVA_HOME"))
+         warning("Java home setting is INVALID, it will be ignored.\nPlease do NOT set it unless you want to override system settings.")
+         javahome <- ""
+    }
+    if(!nzchar(javahome)) { ## JAVA_HOME was not set explicitly
         find.java <- function() {
             for (root in c("HLM", "HCU"))
                 for(key in c(
@@ -23,7 +31,7 @@ function(libname, pkgname) {
             stop("No CurrentVersion entry in Software/JavaSoft registry! Try re-installing Java and make sure R and Java have matching architectures.")
         this <- hive[[hive$CurrentVersion]]
         javahome <- this$JavaHome
-        paths <- dirname(this$RuntimeLib) # wrong on 64-bit
+        paths <- if (is.character(this$RuntimeLib)) dirname(this$RuntimeLib) else character() # wrong on 64-bit
     } else paths <- character()
     if(is.null(javahome) || !length(javahome) || !nchar(javahome))
         stop("JAVA_HOME is not set and could not be determined from the registry")
