@@ -19,24 +19,23 @@ int RJava_request_lock() {
   if (rjctrl && *rjctrl) return 2;
 
   buf[0] = IPCC_LOCK_REQUEST;
-  write(ipcout, buf, sizeof(ptrlong));
+  if (write(ipcout, buf, sizeof(ptrlong)) < sizeof(ptrlong)) return 0;
   n = read(resin, buf, sizeof(ptrlong));
-  return (n > 0 && buf[0] == IPCC_LOCK_GRANTED) ? 1 : 0;
+  return (n == sizeof(ptrlong) && buf[0] == IPCC_LOCK_GRANTED) ? 1 : 0;
 }
 
 int RJava_clear_lock() {
   ptrlong buf[4];
   buf[0] = IPCC_CLEAR_LOCK;
-  write(ipcout, buf, sizeof(ptrlong));
-  return 1;
+  return (write(ipcout, buf, sizeof(ptrlong)) == sizeof(ptrlong)) ? 1 : 0;
 }
 
-void RJava_request_callback(callbackfn *fn, void *data) {
+int RJava_request_callback(callbackfn *fn, void *data) {
   ptrlong buf[4];
   buf[0] = IPCC_CALL_REQUEST;
   buf[1] = (ptrlong) fn;
   buf[2] = (ptrlong) data;
-  write(ipcout, buf, sizeof(ptrlong) * 3);
+  return (write(ipcout, buf, sizeof(ptrlong) * 3) == sizeof(ptrlong) * 3) ? 1 : 0;
 }
 
 void RJava_setup(int _in, int _out) {
@@ -48,9 +47,9 @@ void RJava_setup(int _in, int _out) {
 void RJava_init_ctrl() {
   ptrlong buf[4];
   buf[0] = IPCC_CONTROL_ADDR;
-  write(ipcout, buf, sizeof(ptrlong));
-  read(resin, buf, sizeof(ptrlong) * 2);
-  if (buf[0] == IPCC_CONTROL_ADDR) {
-    rjctrl= (int*) buf[1];
+  if (write(ipcout, buf, sizeof(ptrlong)) == sizeof(ptrlong) &&
+      read(resin, buf, sizeof(ptrlong) * 2) == sizeof(ptrlong) * 2 &&
+      buf[0] == IPCC_CONTROL_ADDR) {
+      rjctrl= (int*) buf[1];
   }
 }
